<?php

/**
 * @file
 * Drush integration for the salsa entity module.
 */

/**
 * Implements hook_drush_command().
 */
function salsa_entity_drush_command() {
  $items['salsa-entity-generate-translatables'] = array(
    'description' => dt('Generates a file with all translatable strings.'),
  );
  return $items;
}

/**
 * Generate a file with all translatable strings.
 */
function drush_salsa_entity_generate_translatables() {
  $types = salsa_entity_object_types();

  $file = fopen(DRUPAL_ROOT . '/' .  drupal_get_path('module', 'salsa_entity') . '/salsa_entity.translatables.inc', 'w');

  $header = '<?php
/**
 * @file
 * This file is automatically generated with drush salsa-generate-translatables.
 */';
  fwrite($file, $header);
  fwrite($file, "\n");
  fwrite($file, "\n");

  foreach ($types as $name => $type) {
    fwrite($file, "// Schema strings for $name.\n");

    $cid = 'describe2:' . $name;
    if (!($salsa_info = salsa_entity_load_cache_item($cid))) {
      $query_info = array(
        '#script' => 'describe2.sjs',
        '#tables' => $name,
      );
      if ($salsa_info = salsa_api_query($query_info)) {
        salsa_entity_save_cache_item($cid, $salsa_info);
      }
    }
    foreach ($salsa_info[$name]['item'] as $item) {
     // Skip some internal/technical things.
     foreach (array('READONLY', 'PRIVATE', 'KEY') as $ignore_pattern) {
       if (strpos($item['name'], $ignore_pattern) !== FALSE) {
         continue 2;
       }
     }
     fwrite($file, "t('" . $item['label'] . "');\n");

     // Write out enum values.
     if ((strpos($item['type'], 'enum') === 0 || strpos($item['type'], 'set') === 0) && !empty($item['values'])) {
       foreach (explode(',', $item['values']) as $value) {
         // Ignore empty values, those with underscores and uppercase values.
         if (!empty($value) && strpos($value, '_') === FALSE && $value != strtoupper($value)) {
           fwrite($file, "t('" . $value . "');\n");
         }
       }
     }
    }
    fwrite($file, "\n");
  }
  fclose($file);
}