<?php

/**
 * @file
 * Definition of SalsaController.
 */


/**
 * Generic Salsa API controller.
 */
class SalsaController extends EntityAPIController implements EntityAPIControllerInterface {

  protected $salsaType;

  public function __construct($entityType) {
    parent::__construct($entityType);
    $this->salsaType = str_replace('salsa_', '', $entityType);
  }

  /*
   * Overridden EntityAPIController::query().
   */
  public function query($ids = array(), $conditions = array(), $revision_id = NULL) {
    if (!empty($ids)) {
      $conditions[$this->salsaType . '_KEY'] = $ids;
    }

    // Handle pseudo conditions.
    $query_info = array();
    if (isset($conditions['#limit'])) {
      $query_info['#limit'] = $conditions['#limit'];
      unset($conditions['#limit']);
    }
    if (isset($conditions['#order'])) {
      $query_info['#order'] = $conditions['#order'];
      unset($conditions['#order']);
    }

    $condition_string = salsa_entity_build_condition_string($conditions);
    $objects = salsa_api_get_objects($this->salsaType, $condition_string, $query_info);
    $entities = array();
    // Salsa entities always have a entity class.
    $entity_class = $this->entityInfo['entity class'];
    foreach ($objects as $key => $object) {
      if (!empty($key)) {
        $entities[$key] = new $entity_class((array)$object, $this->entityType);
      }
    }
    return $entities;
  }

  /*
   * Overridden EntityAPIController::delete().
   */
  public function delete($ids) {
    $entities = $ids ? $this->load($ids) : FALSE;
    if (!$entities) {
      // Do nothing, in case invalid or no ids have been passed.
      return;
    }

    $query_info = array(
      '#script' => 'delete',
      '#tables' => $this->salsaType,
      '#fields' => $ids,
    );
    salsa_api_query($query_info);

    foreach ($entities as $id => $entity) {
      $this->invoke('delete', $entity);
    }
  }

  /*
   * Overridden EntityAPIController::save().
   */
  public function save($entity) {
    $this->invoke('presave', $entity);

    $fields = array();
    $info = entity_get_property_info($this->entityType);
    if (!isset($info['properties'])) {
      debug($this->entityType);
    }
    $property_keys = array_keys($info['properties']);
    // Always allow organization_key and key, which seems to be required
    // sometimes.
    $property_keys[] = 'organization_KEY';
    $property_keys[] = 'key';
    foreach ($property_keys as $key) {
      if (property_exists($entity, $key)) {
        // @todo: Find a better place to put this.
        if ($key == 'Response') {
          $fields['Response_Question_' . $entity->questionnaire_question_KEY] = urlencode($entity->$key);
        }
        else {
          $fields[$key] = urlencode($entity->$key);
        }
      }
    }

    // Support saving/passing additional fields to /save which are not
    // properties.
    if (!empty($entity->additional)) {
      $fields += $entity->additional;
    }

    // Prevent Your password does not match error for supporters.
    // This does not delete the password, it just does not update it.
    unset($fields['Password']);

    $query_info = array(
      '#script' => 'save',
      '#tables' => $this->salsaType,
      '#fields' => $fields,
    );
    $query_result = salsa_api_query($query_info);

    $key = $this->salsaType . '_KEY';
    $is_new = (!isset($entity->$key));
    if ($is_new) {
      $this->invoke('insert', $entity);
    }
    else {
      $this->invoke('update', $entity);
      DrupalDefaultEntityController::resetCache(array($entity->$key));
    }

    if (isset($query_result['error'])) {
      throw new Exception(t('Failed to save entity @type: @error', array('@type' => $this->entityType, '@error' => $query_result['error'])));
    }

    // Reload just saved entity.
    $ID = explode(" ", $query_result['success']);
    $updated_entities = $this->load(array($ID[2]));
    $updated_entity = reset($updated_entities);
    foreach ($updated_entity as $key => $value) {
      $key = trim($key);
      $entity->$key = $value;
    }
  }

}
